import os
import logging
logger = logging.getLogger(__name__)

def setup_tmss_django(dbcreds_id: str):
    '''Setup django for tmss for the given dbcreds_id'''
    os.environ["TMSS_DBCREDENTIALS"] = dbcreds_id
    os.environ["DJANGO_SETTINGS_MODULE"] = "lofar.sas.tmss.tmss.settings"
    import django
    django.setup()

def can_connect_to_database() -> bool:
    '''Check if we can connect to the django tmss database via the object model.
    If not, that usually means that the setup/settings are wrong, or that the database is down.'''
    from django.db.utils import DatabaseError
    from lofar.sas.tmss.tmss.tmssapp.models import Subtask
    try:
        return Subtask.objects.all().count() >= 0
    except DatabaseError:
        return False

def setup_and_check_tmss_django_database_connection(dbcreds_id: str):
    '''Setup django for tmss for the given dbcreds_id and raise RuntimeError if ill-configured'''
    setup_tmss_django(dbcreds_id)

    from lofar.common import dbcredentials
    dbcreds = dbcredentials.DBCredentials().get(dbcreds_id)

    if can_connect_to_database():
        logger.info("Django TMSS up and running using: %s" % dbcreds.stringWithHiddenPassword())
    else:
        raise ConnectionError("Cannot connect to django database via ORM. Check your settings/configuration and if the database is up. Using: %s" % dbcreds.stringWithHiddenPassword())

def setup_and_check_tmss_django_database_connection_and_exit_on_error(dbcreds_id: str = None):
    '''Setup django for tmss for the given dbcreds_id and exit(1) upon any error'''
    try:
        setup_and_check_tmss_django_database_connection(dbcreds_id)
    except Exception as e:
        logger.error(e)
        exit(1)
