"""
This file contains the serializers (for the elsewhere defined data models)
"""

import logging
logger = logging.getLogger(__name__)

from rest_framework import serializers
from .. import models
from .widgets import JSONEditorField
from .common import FloatDurationField, RelationalHyperlinkedModelSerializer, AbstractTemplateSerializer, DynamicRelationalHyperlinkedModelSerializer

class SubtaskStateSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskState
        fields = '__all__'


class SubtaskStateLogSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskStateLog
        fields = '__all__'


class SubtaskTypeSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskType
        fields = '__all__'


class StationTypeSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.StationType
        fields = '__all__'


class AlgorithmSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.Algorithm
        fields = '__all__'


class SubtaskTemplateSerializer(AbstractTemplateSerializer):
    class Meta:
        model = models.SubtaskTemplate
        fields = '__all__'


class DefaultSubtaskTemplateSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultSubtaskTemplate
        fields = '__all__'


class DataproductSpecificationsTemplateSerializer(AbstractTemplateSerializer):
    class Meta:
        model = models.DataproductSpecificationsTemplate
        fields = '__all__'


class DefaultDataproductSpecificationsTemplateSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.DefaultDataproductSpecificationsTemplate
        fields = '__all__'



class DataproductFeedbackTemplateSerializer(AbstractTemplateSerializer):
    class Meta:
        model = models.DataproductFeedbackTemplate
        fields = '__all__'


class SubtaskSerializer(DynamicRelationalHyperlinkedModelSerializer):
    # If this is OK then we can extend API with NO url ('flat' values) on more places if required
    cluster_value = serializers.StringRelatedField(source='cluster', label='cluster_value', read_only=True)
    specifications_doc = JSONEditorField(schema_source='specifications_template.schema')
    duration = FloatDurationField(read_only=True)

    class Meta:
        model = models.Subtask
        fields = '__all__'
        extra_fields = ['cluster_value', 'duration']


class SubtaskInputSerializer(DynamicRelationalHyperlinkedModelSerializer):
    selection_doc = JSONEditorField(schema_source='selection_template.schema')

    class Meta:
        model = models.SubtaskInput
        fields = '__all__'


class SubtaskOutputSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.SubtaskOutput
        fields = '__all__'
        #extra_fields = ['dataproducts', 'consumers']  #TODO: how can we make the inputs and outputs visible in the rest view without making them required for POSTs?


class DataproductSerializer(DynamicRelationalHyperlinkedModelSerializer):
    specifications_doc = JSONEditorField(schema_source='specifications_template.schema')
    feedback_doc = JSONEditorField(schema_source='feedback_template.schema')

    class Meta:
        model = models.Dataproduct
        fields = '__all__'


class AntennaSetSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.AntennaSet
        fields = '__all__'


class DataproductTransformSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductTransform
        fields = '__all__'


class FilesystemSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.Filesystem
        fields = '__all__'


class ClusterSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.Cluster
        fields = '__all__'


class DataproductArchiveInfoSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductArchiveInfo
        fields = '__all__'


class DataproductHashSerializer(DynamicRelationalHyperlinkedModelSerializer):
    class Meta:
        model = models.DataproductHash
        fields = '__all__'


class SAPSerializer(DynamicRelationalHyperlinkedModelSerializer):
    specifications_doc = JSONEditorField(schema_source='specifications_template.schema')

    class Meta:
        model = models.SAP
        fields = '__all__'


class SAPTemplateSerializer(AbstractTemplateSerializer):
    class Meta:
        model = models.SAPTemplate
        fields = '__all__'


class SAPSerializer(DynamicRelationalHyperlinkedModelSerializer):
    specifications_doc = JSONEditorField(schema_source='specifications_template.schema')

    class Meta:
        model = models.SAP
        fields = '__all__'


class SIPidentifierSerializer(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = models.SIPidentifier
        fields = ['unique_identifier', 'source', 'url']

