#!/usr/bin/env python3

# Copyright (C) 2018    ASTRON (Netherlands Institute for Radio Astronomy)
# P.O. Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.

'''
By importing this helper module in your unittest module you get a TMSSTestDatabaseInstance
which is automatically destroyed at the end of the unittest session.
'''

# before we import any django modules the DJANGO_SETTINGS_MODULE and TMSS_DBCREDENTIALS need to be known/set.
# import and start an isolated TMSSTestDatabaseInstance (with fresh database)
# this automagically sets the required  DJANGO_SETTINGS_MODULE and TMSS_DBCREDENTIALS envvars.
# Setup step 1:
from lofar.sas.tmss.test.test_utils import TMSSTestDatabaseInstance
tmss_test_db_instance = TMSSTestDatabaseInstance()
try:
    tmss_test_db_instance.create()
except:
    tmss_test_db_instance.destroy()
    exit(1)

# tell unittest to stop (and automagically cleanup) the test database once all testing is done.
def tearDownModule():
    tmss_test_db_instance.destroy()

# Setup step 2:
# now it's safe to import django modules, and mandatory to set env DJANGO_SETTINGS_MODULE and do django.setup()
import django
django.setup()

# Setup step 3:
# Do the actual tmss  imports
from lofar.sas.tmss.tmss.tmssapp import models

